from typing import Literal

import click

from roll.roll import Roll


class RollParam(click.ParamType):
    name = "roll"

    def convert(
        self, value: str | Roll, param: click.Parameter | None, ctx: click.Context | None
    ) -> Roll | Literal["advantage"] | Literal["disadvantage"]:
        """parse a Roll from a command line string"""
        if isinstance(value, Roll):
            return value
        elif "advantage".startswith(value):
            return "advantage"
        elif "disadvantage".startswith(value):
            return "disadvantage"
        try:
            return Roll.from_str(value)
        except Exception as e:
            self.fail(f"{e}", param, ctx)


ROLL = RollParam()
